@@ -2,8 +2,9 @@
 
 use uucore::error::{UResult, UUsageError};
 
-use clap::{App, Arg};
+use clap::{Arg, Command};
 use selinux::{OpaqueSecurityContext, SecurityClass, SecurityContext};
+use uucore::format_usage;
 
 use std::borrow::Cow;
 use std::ffi::{CStr, CString, OsStr, OsString};
@@ -18,6 +19,9 @@ use errors::{Error, Result, RunconError};
 
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 const ABOUT: &str = "Run command with specified security context.";
+const USAGE: &str = "\
+    {} [CONTEXT COMMAND [ARG...]]
+    {} [-c] [-u USER] [-r ROLE] [-t TYPE] [-l RANGE] COMMAND [ARG...]";
 const DESCRIPTION: &str = "Run COMMAND with completely-specified CONTEXT, or with current or \
                       transitioned security context modified by one or more of \
                       LEVEL, ROLE, TYPE, and USER.\n\n\
@@ -36,26 +40,16 @@ pub mod options {
     pub const RANGE: &str = "range";
 }
 
-fn get_usage() -> String {
-    format!(
-        "{0} [CONTEXT COMMAND [ARG...]]\n    \
-         {0} [-c] [-u USER] [-r ROLE] [-t TYPE] [-l RANGE] COMMAND [ARG...]",
-        uucore::execution_phrase()
-    )
-}
-
-#[uucore_procs::gen_uumain]
+#[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let usage = get_usage();
-
-    let config = uu_app().usage(usage.as_ref());
+    let config = uu_app();
 
     let options = match parse_command_line(config, args) {
         Ok(r) => r,
         Err(r) => {
             if let Error::CommandLine(ref r) = r {
-                match r.kind {
-                    clap::ErrorKind::HelpDisplayed | clap::ErrorKind::VersionDisplayed => {
+                match r.kind() {
+                    clap::ErrorKind::DisplayHelp | clap::ErrorKind::DisplayVersion => {
                         println!("{}", r);
                         return Ok(());
                     }
@@ -73,7 +67,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         CommandLineMode::Print => print_current_context().map_err(|e| RunconError::new(e).into()),
         CommandLineMode::PlainContext { context, command } => {
             get_plain_context(context)
-                .and_then(set_next_exec_context)
+                .and_then(|ctx| set_next_exec_context(&ctx))
                 .map_err(RunconError::new)?;
             // On successful execution, the following call never returns,
             // and this process image is replaced.
@@ -97,7 +91,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                         range.as_deref(),
                         command,
                     )
-                    .and_then(set_next_exec_context)
+                    .and_then(|ctx| set_next_exec_context(&ctx))
                     .map_err(RunconError::new)?;
                     // On successful execution, the following call never returns,
                     // and this process image is replaced.
@@ -109,56 +103,66 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     }
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(uucore::util_name())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .version(VERSION)
         .about(ABOUT)
         .after_help(DESCRIPTION)
+        .override_usage(format_usage(USAGE))
+        .infer_long_args(true)
         .arg(
-            Arg::with_name(options::COMPUTE)
-                .short("c")
+            Arg::new(options::COMPUTE)
+                .short('c')
                 .long(options::COMPUTE)
                 .takes_value(false)
                 .help("Compute process transition context before modifying."),
         )
         .arg(
-            Arg::with_name(options::USER)
-                .short("u")
+            Arg::new(options::USER)
+                .short('u')
                 .long(options::USER)
                 .takes_value(true)
                 .value_name("USER")
-                .help("Set user USER in the target security context."),
+                .help("Set user USER in the target security context.")
+                .allow_invalid_utf8(true),
         )
         .arg(
-            Arg::with_name(options::ROLE)
-                .short("r")
+            Arg::new(options::ROLE)
+                .short('r')
                 .long(options::ROLE)
                 .takes_value(true)
                 .value_name("ROLE")
-                .help("Set role ROLE in the target security context."),
+                .help("Set role ROLE in the target security context.")
+                .allow_invalid_utf8(true),
         )
         .arg(
-            Arg::with_name(options::TYPE)
-                .short("t")
+            Arg::new(options::TYPE)
+                .short('t')
                 .long(options::TYPE)
                 .takes_value(true)
                 .value_name("TYPE")
-                .help("Set type TYPE in the target security context."),
+                .help("Set type TYPE in the target security context.")
+                .allow_invalid_utf8(true),
         )
         .arg(
-            Arg::with_name(options::RANGE)
-                .short("l")
+            Arg::new(options::RANGE)
+                .short('l')
                 .long(options::RANGE)
                 .takes_value(true)
                 .value_name("RANGE")
-                .help("Set range RANGE in the target security context."),
+                .help("Set range RANGE in the target security context.")
+                .allow_invalid_utf8(true),
+        )
+        .arg(
+            Arg::new("ARG")
+                .multiple_occurrences(true)
+                .allow_invalid_utf8(true),
         )
-        .arg(Arg::with_name("ARG").multiple(true))
         // Once "ARG" is parsed, everything after that belongs to it.
         //
         // This is not how POSIX does things, but this is how the GNU implementation
         // parses its command line.
-        .setting(clap::AppSettings::TrailingVarArg)
+        .trailing_var_arg(true)
 }
 
 #[derive(Debug)]
@@ -201,8 +205,8 @@ struct Options {
     arguments: Vec<OsString>,
 }
 
-fn parse_command_line(config: App, args: impl uucore::Args) -> Result<Options> {
-    let matches = config.get_matches_from_safe(args)?;
+fn parse_command_line(config: Command, args: impl uucore::Args) -> Result<Options> {
+    let matches = config.try_get_matches_from(args)?;
 
     let compute_transition_context = matches.is_present(options::COMPUTE);
 
@@ -268,7 +272,7 @@ fn print_current_context() -> Result<()> {
     Ok(())
 }
 
-fn set_next_exec_context(context: OpaqueSecurityContext) -> Result<()> {
+fn set_next_exec_context(context: &OpaqueSecurityContext) -> Result<()> {
     let c_context = context
         .to_c_string()
         .map_err(|r| Error::from_selinux("Creating new context", r))?;
